@@ -692,8 +692,8 @@ void ViewManager::focusAnotherTerminal(ViewSplitter *toplevelSplitter)
 
     if (tabTterminalDisplays.count() > 1) {
         // Give focus to the last used terminal in this tab
-        for (auto *historyItem : _terminalDisplayHistory) {
-            for (auto *terminalDisplay : tabTterminalDisplays) {
+        for (const auto *historyItem : std::as_const(_terminalDisplayHistory)) {
+            for (auto *terminalDisplay : std::as_const(tabTterminalDisplays)) {
                 if (terminalDisplay == historyItem) {
                     terminalDisplay->setFocus(Qt::OtherFocusReason);
                     return;
@@ -845,7 +845,7 @@ SessionController *ViewManager::createController(Session *session, TerminalDispl
     connect(view, &Konsole::TerminalDisplay::destroyed, controller, &Konsole::SessionController::deleteLater);
     connect(controller, &Konsole::SessionController::viewDragAndDropped, this, &Konsole::ViewManager::forgetController);
     connect(controller, &Konsole::SessionController::requestSplitViewLeftRight, this, &Konsole::ViewManager::splitLeftRight);
-    connect(controller, &Konsole::SessionController::requestSplitViewTopBotton, this, &Konsole::ViewManager::splitTopBottom);
+    connect(controller, &Konsole::SessionController::requestSplitViewTopBottom, this, &Konsole::ViewManager::splitTopBottom);
 
     // if this is the first controller created then set it as the active controller
     if (_pluggedController.isNull()) {
@@ -1126,10 +1126,10 @@ QList<ViewProperties *> ViewManager::viewProperties() const
         return {};
     }
 
-    auto terminalContainers = _viewContainer->findChildren<TerminalDisplay *>();
+    const auto terminalContainers = _viewContainer->findChildren<TerminalDisplay *>();
     list.reserve(terminalContainers.size());
 
-    for (auto terminalDisplay : _viewContainer->findChildren<TerminalDisplay *>()) {
+    for (auto terminalDisplay : terminalContainers) {
         list.append(terminalDisplay->sessionController());
     }
 
@@ -1144,6 +1144,10 @@ QJsonObject saveSessionTerminal(TerminalDisplay *terminalDisplay)
     auto terminalSession = terminalDisplay->sessionController()->session();
     const int sessionRestoreId = SessionManager::instance()->getRestoreId(terminalSession);
     thisTerminal.insert(QStringLiteral("SessionRestoreId"), sessionRestoreId);
+    thisTerminal.insert(QStringLiteral("Columns"), terminalDisplay->columns());
+    thisTerminal.insert(QStringLiteral("Lines"), terminalDisplay->lines());
+    thisTerminal.insert(QStringLiteral("WorkingDirectory"), terminalDisplay->session()->currentWorkingDirectory());
+    thisTerminal.insert(QStringLiteral("Command"), QStringLiteral(""));
     return thisTerminal;
 }
 
@@ -1172,11 +1176,14 @@ QJsonObject saveSessionsRecurse(QSplitter *splitter)
 
 void ViewManager::saveLayoutFile()
 {
-    QString fileName(QFileDialog::getSaveFileName(this->widget(),
-                                                  i18nc("@title:window", "Save Tab Layout"),
-                                                  QStringLiteral("~/"),
-                                                  i18nc("@item:inlistbox", "Konsole View Layout (*.json)")));
+    saveLayout(QFileDialog::getSaveFileName(this->widget(),
+                                            i18nc("@title:window", "Save Tab Layout"),
+                                            QStringLiteral("~/"),
+                                            i18nc("@item:inlistbox", "Konsole View Layout (*.json)")));
+}
 
+void ViewManager::saveLayout(QString fileName)
+{
     // User pressed cancel in dialog
     if (fileName.isEmpty()) {
         return;
@@ -1224,12 +1231,43 @@ ViewSplitter *restoreSessionsSplitterRecurse(const QJsonObject &jsonSplitter, Vi
     for (const auto widgetJsonValue : splitterWidgets) {
         const auto widgetJsonObject = widgetJsonValue.toObject();
         const auto sessionIterator = widgetJsonObject.constFind(QStringLiteral("SessionRestoreId"));
+        const auto columnsIterator = widgetJsonObject.constFind(QStringLiteral("Columns"));
+        const auto linesIterator = widgetJsonObject.constFind(QStringLiteral("Lines"));
+        const auto commandIterator = widgetJsonObject.constFind(QStringLiteral("Command"));
+        const auto cwdIterator = widgetJsonObject.constFind(QStringLiteral("WorkingDirectory"));
 
         if (sessionIterator != widgetJsonObject.constEnd()) {
             Session *session = useSessionId ? SessionManager::instance()->idToSession(sessionIterator->toInt()) : SessionManager::instance()->createSession();
 
             auto newView = manager->createView(session);
             currentSplitter->addWidget(newView);
+
+            int columns = newView->columns();
+            int lines = newView->lines();
+            if (columnsIterator != widgetJsonObject.constEnd()) {
+                columns = columnsIterator->toInt();
+            }
+            if (linesIterator != widgetJsonObject.constEnd()) {
+                lines = linesIterator->toInt();
+            }
+            newView->setSize(columns, lines);
+
+            // Set the current working directory if the key is not empty
+            if (cwdIterator != widgetJsonObject.constEnd()) {
+                auto cwd = cwdIterator->toString();
+                if (!cwd.isEmpty()) {
+                    newView->session()->setInitialWorkingDirectory(cwd);
+                }
+            }
+
+            if (commandIterator != widgetJsonObject.constEnd()) {
+                auto command = commandIterator->toString();
+                // Don't open a program that is already running, such as bash
+                if (!command.isEmpty() && command != newView->session()->program()) {
+                    newView->session()->runCommandFromLayout(command);
+                }
+            }
+
         } else {
             auto nextSplitter = restoreSessionsSplitterRecurse(widgetJsonObject, manager, useSessionId);
             currentSplitter->addWidget(nextSplitter);
@@ -1333,7 +1371,7 @@ QStringList ViewManager::sessionList()
     QStringList ids;
 
     for (int i = 0; i < _viewContainer->count(); i++) {
-        auto terminaldisplayList = _viewContainer->widget(i)->findChildren<TerminalDisplay *>();
+        const auto terminaldisplayList = _viewContainer->widget(i)->findChildren<TerminalDisplay *>();
         for (auto *terminaldisplay : terminaldisplayList) {
             ids.append(QString::number(terminaldisplay->sessionController()->session()->sessionId()));
         }
@@ -1461,21 +1499,22 @@ QStringList ViewManager::viewHierarchy()
 
 QList<double> ViewManager::getSplitProportions(int splitterId)
 {
-    auto splitter = _viewContainer->findSplitter(splitterId);
+    const auto *splitter = _viewContainer->findSplitter(splitterId);
     if (splitter == nullptr)
         return QList<double>();
 
+    const QList<int> sizes = splitter->sizes();
     int totalSize = 0;
-    QList<double> percentages;
 
-    for (auto size : splitter->sizes()) {
+    for (const auto& size : sizes) {
         totalSize += size;
     }
 
+    QList<double> percentages;
     if (totalSize == 0)
-        return QList<double>();
+        return percentages;
 
-    for (auto size : splitter->sizes()) {
+    for (auto size : sizes) {
         percentages.append((size / static_cast<double>(totalSize)) * 100);
     }
 
@@ -1561,7 +1600,7 @@ bool ViewManager::createSplitWithExisting(int targetSplitterId, QStringList widg
         ViewSplitter *createdSplitter = new ViewSplitter();
         createdSplitter->setOrientation(horizontalSplit ? Qt::Horizontal : Qt::Vertical);
 
-        for (auto widget : linearLayout) {
+        for (auto widget : std::as_const(linearLayout)) {
             if (auto s = qobject_cast<ViewSplitter *>(widget))
                 createdSplitter->addSplitter(s);
             else
@@ -1606,7 +1645,8 @@ bool ViewManager::resizeSplits(int splitterId, QList<double> percentages)
     int sum = 0;
     QList<int> newSizes;
 
-    for (int size : splitter->sizes()) {
+    const auto sizes = splitter->sizes();
+    for (int size : sizes) {
         sum += size;
     }
 
